@@ -20,12 +20,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <cassert>
 #include <cstdlib>
 #include <fstream>
 #include <iostream>
 #include <memory>
-#include <stdexcept>
 #include <string>
 #include <vector>
 
@@ -36,6 +34,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <rapidjson/istreamwrapper.h>
 
 #include <osmium/geom/coordinates.hpp>
+#include <osmium/handler/check_order.hpp>
 #include <osmium/io/any_input.hpp>
 #include <osmium/io/header.hpp>
 #include <osmium/io/writer_options.hpp>
@@ -48,7 +47,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include "command_extract.hpp"
 #include "exception.hpp"
 
-#include "extract/error.hpp"
 #include "extract/extract_bbox.hpp"
 #include "extract/extract_polygon.hpp"
 #include "extract/geojson_file_parser.hpp"
@@ -108,24 +106,6 @@ namespace {
         throw config_error{"'bbox' member is not an array or object."};
     }
 
-    osmium::Box parse_bbox(const std::string& str) {
-        const auto coordinates = osmium::split_string(str, ',');
-
-        if (coordinates.size() != 4) {
-            throw argument_error{"Need exactly four coordinates in --bbox/-b option."};
-        }
-
-        const osmium::Location bottom_left{std::atof(coordinates[0].c_str()), std::atof(coordinates[1].c_str())};
-        const osmium::Location top_right{std::atof(coordinates[2].c_str()), std::atof(coordinates[3].c_str())};
-
-        if (bottom_left.x() < top_right.x() &&
-            bottom_left.y() < top_right.y()) {
-            return osmium::Box{bottom_left, top_right};
-        }
-
-        throw argument_error{"Need LEFT < RIGHT and BOTTOM < TOP in --bbox/-b option."};
-    }
-
     std::size_t parse_multipolygon_object(const std::string& directory, std::string file_name, std::string file_type, osmium::memory::Buffer& buffer) {
         if (file_name.empty()) {
             throw config_error{"Missing 'file_name' in '(multi)polygon' object."};
@@ -156,8 +136,12 @@ namespace {
             try {
                 OSMFileParser parser{buffer, file_name};
                 return parser();
+            } catch (const std::system_error& e) {
+                throw osmium::io_error{std::string{"While reading file '"} + file_name + "':\n" + e.what()};
             } catch (const osmium::io_error& e) {
                 throw osmium::io_error{std::string{"While reading file '"} + file_name + "':\n" + e.what()};
+            } catch (const osmium::out_of_order_error& e) {
+                throw osmium::io_error{std::string{"While reading file '"} + file_name + "':\n" + e.what()};
             }
         } else if (file_type == "geojson") {
             GeoJSONFileParser parser{buffer, file_name};
@@ -230,6 +214,7 @@ void CommandExtract::parse_config_file() {
 
     std::string directory{get_value_as_string(doc, "directory")};
     if (!directory.empty() && m_output_directory.empty()) {
+        m_vout << "  Directory set to '" << directory << "'.\n";
         set_directory(directory);
     }
 
@@ -242,6 +227,7 @@ void CommandExtract::parse_config_file() {
         throw config_error{"'extracts' member in top-level object must be array."};
     }
 
+    m_vout << "  Reading extracts from config file...\n";
     int extract_num = 1;
     for (const auto& e : json_extracts->value.GetArray()) {
         std::string output;
@@ -255,6 +241,8 @@ void CommandExtract::parse_config_file() {
                 throw config_error{"Missing 'output' field for extract."};
             }
 
+            m_vout << "    Looking at extract '" << output << "'...\n";
+
             std::string output_format{get_value_as_string(e, "output_format")};
             std::string description{get_value_as_string(e, "description")};
 
@@ -262,7 +250,12 @@ void CommandExtract::parse_config_file() {
             const auto json_polygon      = e.FindMember("polygon");
             const auto json_multipolygon = e.FindMember("multipolygon");
 
-            const osmium::io::File output_file{m_output_directory + output, output_format};
+            osmium::io::File output_file{m_output_directory + output, output_format};
+            if (m_with_history) {
+                output_file.set_has_multiple_object_versions(true);
+            } else if (output_file.has_multiple_object_versions()) {
+                throw config_error{"Looks like you are trying to write a history file, but option --with-history is not set."};
+            }
 
             if (json_bbox != e.MemberEnd()) {
                 m_extracts.emplace_back(new ExtractBBox{output_file, description, parse_bbox(json_bbox->value)});
@@ -273,6 +266,22 @@ void CommandExtract::parse_config_file() {
             } else {
                 throw config_error{"Missing geometry for extract. Need 'bbox', 'polygon', or 'multipolygon'."};
             }
+
+            Extract& extract = *m_extracts.back();
+            const auto json_output_header = e.FindMember("output_header");
+            if (json_output_header != e.MemberEnd()) {
+                const auto& value = json_output_header->value;
+                if (!value.IsObject()) {
+                    throw config_error{"Optional 'output_header' field must be an object."};
+                }
+                for (auto it = value.MemberBegin(); it != value.MemberEnd(); ++it) {
+                    const auto& member_value = it->value;
+                    if (!member_value.IsString()) {
+                        throw config_error{"Values in 'output_header' object must be strings."};
+                    }
+                    extract.add_header_option(it->name.GetString(), member_value.GetString());
+                }
+            }
         } catch (const config_error& e) {
             std::string message{"In extract "};
             message += std::to_string(extract_num);
@@ -291,10 +300,14 @@ void CommandExtract::parse_config_file() {
         } catch (const osmium::io_error&) {
             std::cerr << "Error while reading OSM file for extract " << extract_num << " (" << output << "):\n";
             throw;
+        } catch (const osmium::out_of_order_error&) {
+            std::cerr << "Error while reading OSM file for extract " << extract_num << " (" << output << "):\n";
+            throw;
         }
 
         ++extract_num;
     }
+    m_vout << '\n';
 }
 
 std::unique_ptr<ExtractStrategy> CommandExtract::make_strategy(const std::string& name) {
@@ -330,7 +343,8 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
     ("option,S", po::value<std::vector<std::string>>(), "Set strategy option")
     ("polygon,p", po::value<std::string>(), "Polygon file")
     ("strategy,s", po::value<std::string>()->default_value("complete_ways"), "Use named extract strategy")
-    ("with-history", "Input file and output files are history files")
+    ("with-history,H", "Input file and output files are history files")
+    ("set-bounds", "Sets bounds (bounding box) in header")
     ;
 
     po::options_description opts_common{add_common_options()};
@@ -364,15 +378,19 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
         throw argument_error{"Can only use one of --config/-c, --bbox/-b, or --polygon/-p."};
     }
 
+    if (vm.count("with-history")) {
+        m_with_history = true;
+    }
+
     if (vm.count("config")) {
         if (vm.count("directory")) {
             set_directory(vm["directory"].as<std::string>());
         }
         if (vm.count("output")) {
-            warning("Ignoring --output/-o option\n");
+            warning("Ignoring --output/-o option.\n");
         }
         if (vm.count("output-format")) {
-            warning("Ignoring --output-format/-f option\n");
+            warning("Ignoring --output-format/-f option.\n");
         }
         m_config_file_name = vm["config"].as<std::string>();
         auto slash = m_config_file_name.find_last_of('/');
@@ -380,26 +398,19 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
             m_config_directory = m_config_file_name;
             m_config_directory.resize(slash + 1);
         }
-
-        try {
-            parse_config_file();
-        } catch (const config_error&) {
-            std::cerr << "Error while reading config file '" << m_config_file_name << "':\n";
-            throw;
-        }
     }
 
     if (vm.count("bbox")) {
         if (vm.count("directory")) {
-            warning("Ignoring --directory/-d option\n");
+            warning("Ignoring --directory/-d option.\n");
         }
         check_output_file();
-        m_extracts.emplace_back(new ExtractBBox{m_output_file, "", parse_bbox(vm["bbox"].as<std::string>())});
+        m_extracts.emplace_back(new ExtractBBox{m_output_file, "", parse_bbox(vm["bbox"].as<std::string>(), "--box/-b")});
     }
 
     if (vm.count("polygon")) {
         if (vm.count("directory")) {
-            warning("Ignoring --directory/-d option\n");
+            warning("Ignoring --directory/-d option.\n");
         }
         check_output_file();
         m_extracts.emplace_back(new ExtractPolygon{m_output_file, "", m_buffer, parse_multipolygon_object("./", vm["polygon"].as<std::string>(), "", m_buffer)});
@@ -411,8 +422,8 @@ bool CommandExtract::setup(const std::vector<std::string>& arguments) {
         }
     }
 
-    if (vm.count("with-history")) {
-        m_with_history = true;
+    if (vm.count("set-bounds")) {
+        m_set_bounds = true;
     }
 
     if (vm.count("strategy")) {
@@ -436,6 +447,9 @@ void CommandExtract::show_arguments() {
     m_vout << "    output directory: " << m_output_directory << '\n';
 
     m_vout << '\n';
+}
+
+void CommandExtract::show_extracts() {
     m_vout << "Extracts:\n";
 
     int n = 1;
@@ -446,6 +460,18 @@ void CommandExtract::show_arguments() {
         std::cerr.fill(old_fill);
         m_vout << "     Format:      " << e->output_format()    << '\n';
         m_vout << "     Description: " << e->description()      << '\n';
+        if (!e->header_options().empty()) {
+        m_vout << "     Header opts: ";
+            bool first = true;
+            for (const auto& opt : e->header_options()) {
+                if (first) {
+                    first = false;
+                } else {
+                    m_vout << "                  ";
+                }
+                m_vout << opt.first << ": " << opt.second << '\n';
+            }
+        }
         m_vout << "     Envelope:    " << e->envelope_as_text() << '\n';
         m_vout << "     Type:        " << e->geometry_type()    << '\n';
         m_vout << "     Geometry:    " << e->geometry_as_text() << '\n';
@@ -456,11 +482,33 @@ void CommandExtract::show_arguments() {
 }
 
 bool CommandExtract::run() {
+    if (!m_config_file_name.empty()) {
+        m_vout << "Reading config file...\n";
+        try {
+            parse_config_file();
+        } catch (const config_error&) {
+            std::cerr << "Error while reading config file '" << m_config_file_name << "':\n";
+            throw;
+        }
+    }
+
+    show_extracts();
+
     osmium::io::Header header;
     setup_header(header);
 
     for (const auto& extract : m_extracts) {
-        extract->open_file(header, m_output_overwrite, m_fsync);
+        osmium::io::Header file_header{header};
+        if (m_with_history) {
+            file_header.set_has_multiple_object_versions(true);
+        }
+        if (m_set_bounds) {
+            file_header.add_box(extract->envelope());
+        }
+        for (const auto& p : extract->header_options()) {
+            file_header.set(p.first, p.second);
+        }
+        extract->open_file(file_header, m_output_overwrite, m_fsync);
     }
 
     m_strategy->run(m_vout, display_progress(), m_input_file);
