@@ -21,16 +21,12 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
-#include <cerrno>
 #include <cstdint>
 #include <iostream>
 #include <iterator>
 #include <memory>
 #include <sstream>
 #include <string>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <system_error>
 #include <utility>
 #include <vector>
 
@@ -54,6 +50,8 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/osm.hpp>
 #include <osmium/osm/box.hpp>
 #include <osmium/osm/crc.hpp>
+#include <osmium/osm/object_comparisons.hpp>
+#include <osmium/util/file.hpp>
 #include <osmium/util/minmax.hpp>
 #include <osmium/util/progress_bar.hpp>
 #include <osmium/util/verbose_output.hpp>
@@ -74,10 +72,10 @@ struct InfoHandler : public osmium::handler::Handler {
     uint64_t ways       = 0;
     uint64_t relations  = 0;
 
-    osmium::max_op<osmium::object_id_type> largest_changeset_id { 0 };
-    osmium::max_op<osmium::object_id_type> largest_node_id { 0 };
-    osmium::max_op<osmium::object_id_type> largest_way_id { 0 };
-    osmium::max_op<osmium::object_id_type> largest_relation_id { 0 };
+    osmium::max_op<osmium::object_id_type> largest_changeset_id{0};
+    osmium::max_op<osmium::object_id_type> largest_node_id{0};
+    osmium::max_op<osmium::object_id_type> largest_way_id{0};
+    osmium::max_op<osmium::object_id_type> largest_relation_id{0};
 
     osmium::min_op<osmium::Timestamp> first_timestamp;
     osmium::max_op<osmium::Timestamp> last_timestamp;
@@ -114,7 +112,7 @@ struct InfoHandler : public osmium::handler::Handler {
             if (last_id == object.id()) {
                 multiple_versions = true;
             }
-            if (last_id > object.id()) {
+            if (osmium::id_order{}(object.id(), last_id)) {
                 ordered = false;
             }
         } else if (last_type != osmium::item_type::changeset && last_type > object.type()) {
@@ -149,23 +147,6 @@ struct InfoHandler : public osmium::handler::Handler {
 
 }; // struct InfoHandler
 
-/*************************************************************************/
-
-off_t filesize(const std::string& filename) {
-    if (filename.empty()) {
-        return 0;
-    }
-
-    struct stat s;
-    if (::stat(filename.c_str(), &s) == -1) {
-        throw std::system_error{errno, std::system_category(), std::string{"Could not get file size of file '"} + filename + "'"};
-    }
-
-    return s.st_size;
-}
-
-/*************************************************************************/
-
 class Output {
 
 public:
@@ -192,7 +173,7 @@ public:
         std::cout << "  Compression: " << input_file.compression() << "\n";
 
         if (!input_file.filename().empty()) {
-            std::cout << "  Size: " << filesize(input_file.filename()) << "\n";
+            std::cout << "  Size: " << osmium::util::file_size(input_file.filename()) << "\n";
         }
     }
 
@@ -279,7 +260,7 @@ public:
 
         if (!input_file.filename().empty()) {
             m_writer.String("size");
-            m_writer.Int64(filesize(input_file.filename()));
+            m_writer.Int64(osmium::util::file_size(input_file.filename()));
         }
 
         m_writer.EndObject();
@@ -411,7 +392,7 @@ public:
             if (input_file.filename().empty()) {
                 std::cout << 0 << "\n";
             } else {
-                std::cout << filesize(input_file.filename()) << "\n";
+                std::cout << osmium::util::file_size(input_file.filename()) << "\n";
             }
         }
     }
