@@ -24,6 +24,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/util/string.hpp>
 
 #include "strategy_smart.hpp"
+#include "../util.hpp"
 
 namespace strategy_smart {
 
@@ -61,6 +62,12 @@ namespace strategy_smart {
             m_extracts.emplace_back(*extract);
         }
 
+        for (const auto& option : options) {
+            if (std::string{"types"} != option.first) {
+                warning(std::string{"Ignoring unknown option '"} + option.first + "' for 'smart' strategy.\n");
+            }
+        }
+
         const auto types = options.get("types");
         if (types == "") {
             m_types = {"multipolygon"};
@@ -221,7 +228,7 @@ namespace strategy_smart {
 
     void Strategy::run(osmium::util::VerboseOutput& vout, bool display_progress, const osmium::io::File& input_file) {
         vout << "Running 'smart' strategy in three passes...\n";
-        const size_t file_size = osmium::util::file_size(input_file.filename());
+        const std::size_t file_size = osmium::util::file_size(input_file.filename());
         osmium::ProgressBar progress_bar{file_size * 3, display_progress};
 
         vout << "First pass...\n";
@@ -230,7 +237,7 @@ namespace strategy_smart {
         progress_bar.file_done(file_size);
 
         // recursively get parents of all relations that are in an extract
-        auto relations_map = pass1.relations_map_stash().build_index();
+        const auto relations_map = pass1.relations_map_stash().build_member_to_parent_index();
         for (auto& e : m_extracts) {
             for (osmium::unsigned_object_id_type id : e.relation_ids) {
                 e.add_relation_parents(id, relations_map);
